/*+++
 *  title:      MailProxy.m
 *  abstract:   Implementation of MailProxy class for mailapp-utilities.
 *  author:     Tom Hageman <tom@basil.icce.rug.nl>
 *  created:    February 1998
 *  modified:   (see RCS Log at end)
 *  copyleft:
 *
 *	Copyright (C) 1998  Tom R. Hageman, but otherwise perfect freeware.
 *
 *  description:
 *      (see corresponding *.h file)
 *---*/

static const char * const RCSid = ((void)&RCSid,
	"@(#)MailProxy.m,v 1.5 1998/08/12 02:32:35");
#define RCS_MailProxy_ID

#import "compat.h"
#import "mailutil.h"
#import <stdlib.h>
#import <mach/mach.h>

#define _MailProxy_common_ivars \
   char *hostname; \
   char *mailer; \
   BOOL isConnected;

#define _RELEASE_COMMON_IVARS \
   free(hostname); \
   free(mailer);

#if RHAPSODY

#  import <AppKit/NSWorkspace.h>
#  import <Message/NSMailDelivery.h>

// Undocumented DO protocol from MailViewer
@protocol MVDOServer <NSObject>
- (oneway void)newComposeWindowWithHeaders:(bycopy NSDictionary *)headers
	body:(bycopy NSAttributedString *)body;
@end

#  define DO_SERVER_NAME_FORMAT	@"%s DO Server-%@" // mailer, username

#  define _MailProxy_ivars \
   _MailProxy_common_ivars \
   id <MVDOServer> server;

#  define _RELEASE_IVARS \
   _RELEASE_COMMON_IVARS \
   [server release];

#else // NEXTSTEP || OPENSTEP

#  import "MailSpeaker.h"

#  define _MailProxy_ivars \
   _MailProxy_common_ivars \
   MailSpeaker *mailSpeaker;

#  if NEXTSTEP
#     define _RELEASE_IVARS \
   _RELEASE_COMMON_IVARS \
   [mailSpeaker free];
#  else // OPENSTEP
#     define _RELEASE_IVARS \
   _RELEASE_COMMON_IVARS \
   [mailSpeaker release];
#  endif

#endif

#import "MailProxy.h"


@implementation MailProxy

// Implementation feature tests.
+ (BOOL)canEditMessage
{
#if RHAPSODY
   return YES; // was NO;
#else
   return YES;
#endif
}

+ (BOOL)canIncorporateNewMail
{
#if RHAPSODY
   return NO;
#else
   return YES;
#endif
}


- initWithMailer:(const char *)aMailer
   host:(const char *)aHostName
   forceLaunch:(BOOL)forceLaunch
{
   if ((self = [super init]))
   {
      if (aHostName == NULL) aHostName = "";
      hostname = strcpy(malloc(strlen(aHostName)+1), aHostName);
      // XXX find name of local host?

      if (!aMailer) aMailer = "";

#if RHAPSODY
      if (!*aMailer) aMailer = NXGetDefaultValue("GLOBAL", "Mailer");
      if (!aMailer || !*aMailer) aMailer = MAIL_APP;
      {
	 NSString *name = [NSString stringWithFormat:DO_SERVER_NAME_FORMAT,
			      aMailer, NSUserName()];
	 NSString *host = [NSString stringWithCString:hostname];

	 server = (id <MVDOServer>)[NSConnection rootProxyForConnectionWithRegisteredName:name host:host];

	 if (server == nil && forceLaunch)
	 {
	    int i;

	    /* Currently only supports launching Mailer app on local host. */
	    if (strcmp(hostname, "") != 0)
	    {
	       [self release];
	       return nil;
	    }
	    [[NSWorkspace sharedWorkspace] launchApplication:[NSString stringWithCString:aMailer]];

	    // Give it some time to launch itself.
	    for (i = 5;  i > 0;  --i)
	    {
	       server = (id <MVDOServer>)[NSConnection rootProxyForConnectionWithRegisteredName:name host:host];
	       if (server) break;
	       [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1]];
	    }
	 }
      }
      if (server)
      {
	 [server retain];
	 [(NSDistantObject *)server setProtocolForProxy:@protocol(MVDOServer)];
	 isConnected = YES;
      }
#else
      mailSpeaker = [[MailSpeaker alloc] init];
      {
	 port_t mailPort = NXPortNameLookup(MAILSEND_PORTNAME, hostname);

	 if (mailPort == PORT_NULL && forceLaunch)
	 {
	    if ((mailPort = NXPortFromName(aMailer, hostname)) == PORT_NULL)
	    {
	       if (!*aMailer) aMailer = NXGetDefaultValue("GLOBAL", "Mailer");
	       if (!aMailer || !*aMailer) aMailer = MAIL_APP;
	       mailPort = NXPortFromName(aMailer, hostname);
	       if (mailPort == PORT_NULL)
	       {
		  aMailer = MAIL_APP;
		  mailPort = NXPortFromName(aMailer, hostname);
	       }
	       if (mailPort != PORT_NULL) port_deallocate(task_self(), mailPort);
	    }
	    mailPort = NXPortNameLookup(MAILSEND_PORTNAME, hostname);
	 }
	 if (mailPort != PORT_NULL)
	 {
	    [mailSpeaker setSendPort:mailPort];
	    isConnected = YES;
	 }
      }
#endif
      mailer = strcpy(malloc(strlen(aMailer)+1), aMailer);
   }
   return self;
}

#if NEXTSTEP
- free
{
   _RELEASE_IVARS
   return [super free];
}

- (void)release { [self free]; }
#else
- (void)dealloc
{
    _RELEASE_IVARS
    [super dealloc];
}
#endif


// Access methods.

- (const char *)hostname   { return hostname; }
- (const char *)mailer	   { return mailer; }
- (BOOL)isConnected	   { return isConnected; }

// Other methods.

#if RHAPSODY
- (BOOL)deliverBody:(NSAttributedString *)body headers:(NSDictionary *)headers
{
   return [NSMailDelivery deliverMessage:body headers:headers
	   format:NSASCIIMailFormat protocol:nil];
}

- (BOOL)editBody:(NSAttributedString *)body headers:(NSDictionary *)headers
{
   NS_DURING
   {
      [server newComposeWindowWithHeaders:headers body:body];
   }
   NS_HANDLER
   {
      return NO;
   }
   NS_ENDHANDLER
   return isConnected;
}
#endif

- (BOOL)sendMailTo:(const char *)to
   subject:(const char *)subject
   body:(const char *)body
   cc:(const char *)cc 
   deliver:(BOOL)deliver
{
#if RHAPSODY
   NSMutableDictionary *headers = [NSMutableDictionary dictionary];
   NSAttributedString *messageBody = nil;

   if (to)      [headers setObject:[NSString stringWithCString:to]      forKey:@"To"];
   if (subject)	[headers setObject:[NSString stringWithCString:subject] forKey:@"Subject"];
   if (cc)	[headers setObject:[NSString stringWithCString:cc]      forKey:@"Cc"];
   if (body)
   {
      messageBody = [[[NSAttributedString alloc]
		      initWithString:[NSString stringWithCString:body]] autorelease];
   }
   return (deliver) ? [self deliverBody:messageBody headers:headers] :
		      [self editBody:messageBody headers:headers];
#else
   int status = 0;
   int window;

   if ([mailSpeaker openSend:&window] != 0) return NO;

   if (to)	status |= [mailSpeaker setTo:(char *)to		  inWindow:window];
   if (subject)	status |= [mailSpeaker setSubject:(char *)subject inWindow:window];
   if (cc)	status |= [mailSpeaker setCc:(char *)cc		  inWindow:window];
   if (body)	status |= [mailSpeaker setBody:(char *)body	  inWindow:window];

   if (deliver)
   {
      if (to && (subject || body)) status |= [mailSpeaker deliver:window];
   }
   return (status == 0);
#endif
}

- (BOOL)incorporateNewMail
{
#if RHAPSODY
   return NO;
#else
   return [mailSpeaker incorporateNewMail] == 0;
#endif
}

@end // MailProxy
